package bank;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * @author S.Chassande-Barrioz
 */
public class Agency {
    protected String name;
    protected Collection clients;
    protected int lastAccountNumber;
    protected Bank bank;

    public Agency() {
    }

    public Agency(String name, Bank bank) {
        this.name = name;
        this.bank = bank;
        lastAccountNumber = 0;
        clients = new ArrayList();
    }

    public Client getClient(String fn,
                            String ln,
                            String ad) {
        Iterator it = clients.iterator();
        while(it.hasNext()) {
            Client c = (Client) it.next();
            if (c.firstName.equals(fn)
                && c.lastName.equals(ln) && c.address.equals(ad) ) {
                return c;
            }
        }
        return null;
    }

    public synchronized Client createClient(String fn,
                                            String ln,
                                            String ad) {
        Client client = getClient(fn, ln, ad);
        if (client == null) {
            client = new Client(ln, fn, ad, this);
        }
        return client;
    }

    public void removeClient(Client c) {
        if (clients.remove(c)) {
            c.agency = null;
        }
    }

    public void removeAllClients() {
        Iterator it = clients.iterator();
        while (it.hasNext()) {
            Client a = (Client) it.next();
            a.agency = null;
        }
        clients.clear();
    }

    public synchronized String getNextAccountNumber() {
        lastAccountNumber++;
        return getStringAccountNumber(lastAccountNumber);
    }

    public static String getStringAccountNumber(int an) {
        String str = String.valueOf(an);
        for(int i=str.length(); i<11; i++) {
            str = '0' + str;
        }
        return str;
    }
}
