/**
 * Copyright (C) 2001-2006 France Telecom R&D
 */
package inheritance.filtered.disc_in_pk;

import org.objectweb.util.monolog.api.BasicLevel;

import common.MainHelper;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.jdo.PersistenceManager;
import javax.jdo.Query;

public class Main extends MainHelper {

    public static void main(String[] args) {
        new Main().init(args).run().end();
    }
    
    public MainHelper run() {
        E e1 = new E();
        e1.setMyA(new A("a1"));
        e1.setMyC(new C("c1"));
        e1.setMyD(new D("d1"));
        
        E e2 = new E();
        e2.setMyA(new D("d2"));
        e2.setMyC(new D("d3"));
        e2.setMyD(new D("d4"));
        PersistenceManager pm = pmf.getPersistenceManager();
        pm.currentTransaction().begin();
        pm.makePersistent(e1);
        Object oide1 = pm.getObjectId(e1);
        pm.makePersistent(e2);
        Object oide2 = pm.getObjectId(e2);
        e1 = null;
        e2 = null;
        pm.currentTransaction().commit();

        pm.evictAll();
        
        pm.currentTransaction().begin();
        e1 = (E) pm.getObjectById(E.class, oide1);
        logger.log(BasicLevel.INFO, "e1.myA.aid=" + e1.getMyA().getAid());
        logger.log(BasicLevel.INFO, "e1.myC.aid=" + e1.getMyC().getAid());
        logger.log(BasicLevel.INFO, "e1.myD.aid=" + e1.getMyD().getAid());
        e1 = null;
        e2 = (E) pm.getObjectById(E.class, oide2);
        logger.log(BasicLevel.INFO, "e2.myA.aid=" + e2.getMyA().getAid());
        logger.log(BasicLevel.INFO, "e2.myC.aid=" + e2.getMyC().getAid());
        logger.log(BasicLevel.INFO, "e2.myD.aid=" + e2.getMyD().getAid());
        e2 = null;
        pm.currentTransaction().commit();
        
        pm.evictAll();
        
        pm.currentTransaction().begin();
        Query q = pm.newQuery(A.class);
        Collection c = (Collection) q.execute();
        // expected "a1", "c1", "d1", "d2", "d3", "d4"
        logger.log(BasicLevel.INFO, "Search all A (including C and D instances):");
        for (Iterator it = c.iterator(); it.hasNext();) {
            A a = (A) it.next();
            logger.log(BasicLevel.INFO, "\t- " + a.getAid());
        }
        q.closeAll();
        pm.currentTransaction().commit();

        pm.currentTransaction().begin();
        q = pm.newQuery(A.class);
        c = new ArrayList((Collection) q.execute());
        q.closeAll();
        q = pm.newQuery(E.class);
        c.addAll((Collection) q.execute());
        q.closeAll();
        pm.deletePersistentAll(c);
        pm.currentTransaction().commit();
        
        pm.close();
        return this;
    }

}
