/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.enhancer.common;

import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.Constants;
import org.objectweb.asm.Type;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.util.lib.StringReplace;
import org.objectweb.speedo.lib.Personality;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class LoggedClassAdapter extends ClassAdapter implements Constants{
	protected Personality personality;
    protected Logger logger;
    protected boolean debug = false;

    public LoggedClassAdapter(ClassVisitor classVisitor, Personality p) {
        super(classVisitor);
    	personality = p;
    }

    public LoggedClassAdapter(ClassVisitor classVisitor, Personality p, Logger logger) {
        super(classVisitor);
    	personality = p;
        this.logger = logger;
        if (logger != null) {
            debug = logger.isLoggable(BasicLevel.DEBUG);
        }
    }

    public Logger getLogger() {
        return logger;
    }

    public void setLogger(Logger logger) {
        this.logger = logger;
    }
    
    public final static String getJVMClassName(String className) {
        return StringReplace.replaceChar('.', '/', className);
    }

    public final static String getJVMClassName(java.lang.Class clazz) {
        return getJVMClassName(clazz.getName());
    }

    public final static String getJVMType(java.lang.Class clazz) {
        return getJVMType(clazz.getName());
    }

    public final static String getJVMType(String className) {
        return "L" + getJVMClassName(className) + ";";
    }

    public final static String getJVMType(PType pt) {
        return Type.getType(pt.getJavaClass()).getDescriptor();
    }
}
