/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.speedo.generation.jdo;

import org.objectweb.speedo.generation.AbstractEnhancer;
import org.objectweb.speedo.generation.api.GeneratorComponent;
import org.objectweb.speedo.generation.compiler.Compiler;
import org.objectweb.speedo.generation.enhancer.aware.PersistenceAwareEnhancer;
import org.objectweb.speedo.generation.enhancer.pc.jdo.JDOPersistentClassEnhancer;
import org.objectweb.speedo.generation.generator.jdo.JDOGenerator;
import org.objectweb.speedo.generation.jorm.JormGenerator;
import org.objectweb.speedo.generation.jorm.JormMIGenerator;
import org.objectweb.speedo.generation.mivisitor.JMICleanerVisitor;
import org.objectweb.speedo.generation.mivisitor.ORMappingGenerator;
import org.objectweb.speedo.generation.mivisitor.jdo.JDOSMIVisitor;
import org.objectweb.speedo.generation.parser.jdo.JDO2Parser;
import org.objectweb.speedo.generation.recompiler.Recompiler;
import org.objectweb.speedo.generation.recompiler.UpToDateVisitor;
import org.objectweb.speedo.generation.serializer.MISerializer;
import org.objectweb.speedo.lib.Personality;

/**
 * This class is the enhancer for the JDO personality of Speedo.
 * 
 * @author S.Chassande-Barrioz
 */
public class JDOEnhancer extends AbstractEnhancer {

    /**
     * Defines the different step of the Speedo JDO compiler/enhancer
     */
    private static final GeneratorComponent[] GC_GENCOMPS = new GeneratorComponent[] {
        new JDO2Parser(),
        new UpToDateVisitor(Personality.JDO),
        new Recompiler(Personality.JDO),
        new JDOSMIVisitor(),
        new ORMappingGenerator(Personality.JDO),
        new JormMIGenerator(Personality.JDO),
        new JormGenerator(Personality.JDO),
        new JDOGenerator(Personality.JDO),
        new MISerializer(Personality.JDO),
        new JDOPersistentClassEnhancer(),
        new PersistenceAwareEnhancer(Personality.JDO),
        new JMICleanerVisitor(Personality.JDO),
        new Compiler(Personality.JDO),
    };

    protected GeneratorComponent[] getGeneratorComponents() {
        return GC_GENCOMPS;
    }

    public JDOEnhancer() {
    	super(Personality.JDO);
    }
}
