/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.generation.jorm;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.compiler.api.JormCompilerConfigurator;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.compiler.lib.JormCompiler;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.util.io.lib.DirJavaExplorer;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.api.SpeedoXMLError;
import org.objectweb.speedo.generation.lib.AbstractGeneratorComponent;
import org.objectweb.speedo.lib.Personality;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.speedo.metadata.SpeedoModifier;
import org.objectweb.speedo.metadata.SpeedoPackage;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Generates the JORM Files corresponding to a set of Java Classes.
 * <p>See <a href="http://www.objectweb.org/jorm">http://www.objectweb.org/jorm</a>for more information.</p>
 * @author	S.Chassande-Barrioz
 */
public class JormGenerator extends AbstractGeneratorComponent {
    public final static String LOGGER_NAME
            = SpeedoProperties.LOGGER_NAME + ".generation.jorm";
    /**
     * Jorm classes intended for JORM generation
     */
    protected JormCompiler jormcompiler;

    public JormGenerator(Personality p) {
    	super(p);
    }

    // IMPLEMENTATION OF THE GeneratorComponent INTERFACE //
    //----------------------------------------------------//

    public boolean init() throws SpeedoException {
	    if (scp.getXmldescriptor().isEmpty()) {
		    return false;
	    }
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
        jormcompiler = (JormCompiler) scp.generatorsContext.get("jormcompiler");
        JormCompilerParameter jcp = jormcompiler.getCompilerParameter();
        jcp.setProjectName(scp.projectName);
        jcp.setBindingInheritance("%p%c");
        jcp.setClassMappingInheritance("%p%cHome");
        jcp.setOutput(scp.output);
        jcp.setBindingAbstract(true);
        jcp.setDtdLocations(scp.dtdLocations);
        jcp.setInputFiles(scp.jorm);
        DirJavaExplorer pe = new DirJavaExplorer();
        pe.setLogger(scp.loggerFactory.getLogger("org.objectweb.jorm.io.pathexplorer"));
        pe.addPath(scp.jormclasspath);
        jcp.setClasspath(pe);
        jcp.setGenerateBinding(false);
        jcp.setGeneratedWithMapperPackage(false);
	    return true;
    }
    public String getTitle() {
        return "Generating Jorm files...";
    }

    /**
     * Performs the JORM objects' generation.
     */
    public void process() throws SpeedoException {
        if (scp.getXmldescriptor().isEmpty()) {
	        return;
        }
        if (logger.isLoggable(BasicLevel.DEBUG)) {
            Manager m = jormcompiler.getMIManager();
            for (Iterator it = m.getClasses().iterator(); it.hasNext();) {
                logger.log(BasicLevel.DEBUG, "Class : "
                        + ((Class) it.next()).getFQName());
            }
            for (Iterator it = m.getCompositeNames().iterator(); it.hasNext();) {
                logger.log(BasicLevel.DEBUG, "CompositeName : "
                        + ((CompositeName) it.next()).getFQName());
            }
        }
	    try {
			// Generate the .pd files
            if (scp.generateNeededJormFile) {
                jormcompiler.generateJormFiles(null);
            }
            //Launch the jorm generation
            Collection gfs = jormcompiler.generateFiles(null);
            if (logger.isLoggable(BasicLevel.DEBUG))
                for (Iterator it = gfs.iterator(); it.hasNext();)
                    logger.log(BasicLevel.DEBUG, "Generated files: " + it.next());

            //Register the meta information to serialize for each xml descriptor
            for (Iterator itDesc = scp.getXmldescriptor().values().iterator(); itDesc.hasNext();) {
                addSerializeJormMI((SpeedoXMLDescriptor) itDesc.next());
            }
        } catch (PException e) {
            throw new SpeedoException("Error during Jorm generation", e);
        }
    }

    private void addSerializeJormMI(SpeedoXMLDescriptor desc)
            throws SpeedoException {
        // Calculate the list of the Jorm Meta Object which must be serialized
        //Set mos = new HashSet(desc.jdoPackage.size());
        for (Iterator itPack = desc.packages.values().iterator(); itPack.hasNext();) {
            SpeedoPackage sp = (SpeedoPackage) itPack.next();
            for (Iterator itclass = sp.classes.values().iterator(); itclass.hasNext();) {
                SpeedoClass jdoClass = (SpeedoClass) itclass.next();
                if (jdoClass.jormclass == null)
                    throw new SpeedoException(
                            "The Jorm meta information of the jdo descriptor "
                            + desc.xmlFile + " is not complete: class "
                            + jdoClass.name + " has not been found");
                // The class meta object
                addMOClass(jdoClass.jormclass, desc.mos);

                // Add the CompositeName which could be used in the class name def
                for (Iterator it = jdoClass.jormclass.getNameDefs().iterator(); it.hasNext();) {
                    NameDef nd = (NameDef) it.next();
                    if (nd.isNameRef()) {
                        desc.mos.add(nd.getNameRef().getCompositeName());
                    }
                }

                //The compositeName used for the reference will be added by the
                // name definition of the referenced class.
            }
        }
    }

    private void addMOClass(Class mo, Set mos) {
        mos.add(mo);
        for(Iterator it = mo.getSuperClasses().iterator();it.hasNext();) {
            addMOClass((Class) it.next(), mos);
        }
    }
}
