/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.speedo.generation.start;

import org.objectweb.speedo.generation.api.SpeedoCompilerParameter;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;


/**
 * Searches resources among a set of files.
 */
public class ListResourceLocator
    extends ResourceLocatorBase
    implements ResourceLocator
{
    /**
     * The map of jdo files.
     */
    final ArrayList files = new ArrayList();

    /**
     * Creates an intsance.
     */
    public ListResourceLocator(PrintWriter out,
                               boolean verbose,
                               List fileNames,
                               String dir,
                               SpeedoCompilerParameter scp)
        throws IOException
    {
     	super(out, verbose);
        Assertion.assertion(fileNames != null);
		
        // hash the file objects by their canonical name
        for (Iterator i = fileNames.iterator(); i.hasNext();) {
            final String s = (String)i.next();

            // canonicalize file name
            final File file = new File(dir+File.separator+s);
            final URL url = file.toURL();
            final String canonicalName = url.toString();

            Assertion.assertion(canonicalName != null);
            // ensure file is readable
            if (!file.canRead()) {
                final String msg = getI18N("enhancer.cannot_read_resource", file.toString());
                throw new IOException(msg);
            }

           	files.add(file.toString().substring(dir.length()+1));


            printMessage(getI18N("enhancer.using_resource",
                                 canonicalName));
        }
    }
    
	public Collection getCollectionOfFiles() {
		ArrayList res = new ArrayList();
		for (Iterator i = files.iterator();i.hasNext();){
			String f = (String) i.next();
			res.add(f);
		}
		return res;
	}
    
    
    /**
     * Finds a resource with a given name.
     */
    public InputStream getInputStreamForResource(String resourceName)
    {
 		return null;    // unused
    }
}
