/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package org.objectweb.speedo.generation.start;

import java.util.ResourceBundle;
import java.util.Locale;
import java.util.Hashtable;
import java.text.MessageFormat;


class I18NHelper
{
    private static Hashtable	bundles = new Hashtable();
    private static Locale  	locale = Locale.getDefault();

    /**
     * Constructor
     */
    public I18NHelper() 
    {}

    /**
     * Load ResourceBundle by bundle name
     */
    public static ResourceBundle loadBundle(String bundleName)
    {
        ResourceBundle messages = (ResourceBundle)bundles.get(bundleName);

        if (messages == null) //not found as loaded - add
        {
            messages = ResourceBundle.getBundle(bundleName, locale);
            bundles.put(bundleName, messages);
        }
        return messages;
    }

  
    /**
     * Returns message as String
     */
    final public static String getMessage(ResourceBundle messages, String messageKey) 
    {
        return messages.getString(messageKey);
    }

    /**
     * Formats message by adding Array of arguments
     */
    final public static String getMessage(ResourceBundle messages, String messageKey, Object msgArgs[]) 
    {
        for (int i=0; i<msgArgs.length; i++) {
            if (msgArgs[i] == null) msgArgs[i] = ""; // NOI18N
        }
        MessageFormat formatter = new MessageFormat(messages.getString(messageKey));
        return formatter.format(msgArgs);
    }
    /**
     * Formats message by adding a String argument
     */
    final public static String getMessage(ResourceBundle messages, String messageKey, String arg) 
    {
        Object []args = {arg};
        return getMessage(messages, messageKey, args);
    }
    /**
     * Formats message by adding two String arguments
     */
    final public static String getMessage(ResourceBundle messages, String messageKey, String arg1,
                                          String arg2) 
    {
        Object []args = {arg1, arg2};
        return getMessage(messages, messageKey, args);
    }
    /**
     * Formats message by adding three String arguments
     */
    final public static String getMessage(ResourceBundle messages, String messageKey, String arg1,
                                          String arg2, String arg3) 
    {
        Object []args = {arg1, arg2, arg3};
        return getMessage(messages, messageKey, args);
    }
    /**
     *
     * Formats message by adding an Object as an argument
     */
    final public static String getMessage(ResourceBundle messages, String messageKey, Object arg) 
    {
        Object []args = {arg};
        return getMessage(messages, messageKey, args);
    }
    /**
     * Formats message by adding an int as an argument
     */
    final public static String getMessage(ResourceBundle messages, String messageKey, int arg) 
    {
        Object []args = {new Integer(arg)};
        return getMessage(messages, messageKey, args);
    }
    /**
     * Formats message by adding a boolean as an argument
     */
    final public static String getMessage(ResourceBundle messages, String messageKey, boolean arg) 
    {
        Object []args = {String.valueOf(arg)};
        return getMessage(messages, messageKey, args);
    }
}


/**
 * Basic support for enhancer implementation.
 */
public class Support
    extends Assertion
{
    //^olsen: hack
    static public Timer timer = new Timer();

    /**
     * I18N message handler
     */
    static private ResourceBundle MESSAGES;


    /**
     *
     */
    static
    {
        try
        {
            MESSAGES = I18NHelper.loadBundle("org.objectweb.speedo.generation.start.Bundle");
        }
        catch (java.util.MissingResourceException ex)
        {
            ex.printStackTrace ();
        }
    }

    /**
     * Returns the I18N message.
     */
    static protected final String getI18N(String key)
    {
        return I18NHelper.getMessage(MESSAGES, key);
    }

    /**
     * Returns the I18N message.
     */
    static protected final String getI18N(String key,
                                          String arg)
    {
        return I18NHelper.getMessage(MESSAGES, key, arg);
    }

    /**
     * Returns the I18N message.
     */
    static protected final String getI18N(String key,
                                          String arg1,
                                          String arg2)
    {
        return I18NHelper.getMessage(MESSAGES, key, arg1, arg2);
    }

    /**
     * Returns the I18N message.
     */
    static protected final String getI18N(String key,
                                          String arg1,
                                          String arg2,
                                          String arg3)
    {
        return I18NHelper.getMessage(MESSAGES, key, arg1, arg2, arg3);
    }

    /**
     * Returns the I18N message.
     */
    static protected final String getI18N(String key,
                                          int arg1,
                                          String arg2)
    {
        return I18NHelper.getMessage(MESSAGES, key,
                                     new Object[]{new Integer(arg1), arg2});
    }

    /**
     * Returns the I18N message.
     */
    static protected final String getI18N(String key,
                                          Object[] args)
    {
        return I18NHelper.getMessage(MESSAGES, key, args);
    }
}
