/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.jmx.mbeans;

import org.objectweb.perseus.pool.api.Pool;
import org.objectweb.perseus.pool.api.PoolAttributes;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.pm.api.POManagerFactoryItf;

import java.util.Collection;

/**
 * 
 *
 * @author S.Chassande-Barrioz
 */
public class Connection implements ConnectionMBean {

	private POManagerFactoryItf pmf;
	private PoolAttributes pa;
	private Pool pool;

    public Connection(PoolAttributes pa, Pool pool, POManagerFactoryItf pmf) {
        this.pa = pa;
        this.pool = pool;
		this.pmf = pmf;
    }

    // IMPLEMENTS THE ConnectionMBean INTERFACE //
    //------------------------------------------//

    public int getPool_Of_Connection_MaxSize() {
        return pa.getMaxSize();
    }
    public void setPool_Of_Connection_MaxSize(int arg0) throws Exception {
        pa.setMaxSize(arg0);
    }

    public int getPool_Of_Connection_MinSize() {
        return pa.getMinSize();
    }
    public void setPool_Of_Connection_MinSize(int arg0) throws Exception {
        pa.setMinSize(arg0);
    }

    public long getPool_Of_Connection_Timeout() {
        return pa.getTimeout();
    }
    public void setPool_Of_Connection_Timeout(long arg0) {
        pa.setTimeout(arg0);
    }

    public long getPool_Of_Connection_TTL() {
        return pa.getTTL();
    }
    public void setPool_Of_Connection_TTL(long arg0) {
        pa.setTTL(arg0);
    }

    public int getPool_Of_Connection_Current_Size() {
        return pool.getSize();
    }

    public int getPool_Of_Connection_Current_Free_Number() {
        return pool.getFreeResourceNumber();
    }
    
    public int getPool_Of_Connection_Current_Used_Number() {
        return pool.getUsedResourceNumber();
    }
    
    public Collection getPool_Of_Connection_User() {
        return pool.getUsers();
    }
    
	public String getConnectionFactoryName() {
		return pmf.getProperties().getProperty(SpeedoProperties.JDO_OPTION_CONNECTION_FACTORY_NAME);
	}
	public String getJDBC_Connection_DriverClass() {
		return pmf.getProperties().getProperty(SpeedoProperties.JDO_OPTION_CONNECTION_DRIVER_NAME);
	}
	public String getJDBC_Connection_URL(){
		return pmf.getProperties().getProperty(SpeedoProperties.JDO_OPTION_CONNECTION_URL);
	}
	public String getJDBC_Connection_User(){
		return pmf.getProperties().getProperty(SpeedoProperties.JDO_OPTION_CONNECTION_USER_NAME);
	}
	public String getJDBC_Connection_MapperName(){
		return pmf.getProperties().getProperty(SpeedoProperties.MAPPER_NAME);
	}
}
