/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.speedo.jmx.mbeans;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.perseus.pool.api.Pool;
import org.objectweb.perseus.pool.api.PoolAttributes;
import org.objectweb.speedo.AbstractSpeedo;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.mapper.api.JormFactory;
import org.objectweb.speedo.mapper.api.JormFactoryAttributes;
import org.objectweb.speedo.pm.jdo.api.JDOPOManagerFactoryItf;

/**
 * 
 *
 * @author chassase
 */
public class PMF implements PMFMBean {

    private JDOPOManagerFactoryItf pmf;
    private PMapper mapper;
    private JormFactory jf;
    private JormFactoryAttributes jfa;
    private PoolAttributes pa;
    private Pool pool;
    
    public PMF(PoolAttributes pa,
            Pool pool,
            JDOPOManagerFactoryItf pmf, 
            PMapper mapper, 
            JormFactory jf,
            JormFactoryAttributes jfa) {
        this.pa = pa;
        this.pool = pool;
        this.pmf = pmf;
        this.mapper = mapper;
        this.jf = jf;
        this.jfa = jfa;
    }
    
    public int getPool_Of_PM_MaxSize() {
        return pa.getMaxSize();
    }
    public void setPool_Of_PM_MaxSize(int arg0) throws Exception {
        pa.setMaxSize(arg0);
    }

    public int getPool_Of_PM_MinSize() {
        return pa.getMinSize();
    }
    public void setPool_Of_PM_MinSize(int arg0) throws Exception {
        pa.setMinSize(arg0);
    }

    public long getPool_Of_PM_Timeout() {
        return pa.getTimeout();
    }
    public void setPool_Of_PM_Timeout(long arg0) {
        pa.setTimeout(arg0);
    }

    public long getPool_Of_PM_TTL() {
        return pa.getTTL();
    }
    public void setPool_Of_PM_TTL(long arg0) {
        pa.setTTL(arg0);
    }

    public int getPool_Of_PM_Current_Size() {
        return pool.getSize();
    }

    public int getPool_Of_PM_Current_Free_Number() {
        return pool.getFreeResourceNumber();
    }
    public int getPool_Of_PM_Current_Used_Number() {
        return pool.getUsedResourceNumber();
    }

    public boolean getTransaction_RetainValues() {
		return pmf.getRetainValues();
    }
    public void setTransaction_RetainValues(boolean rv) {
		pmf.setRetainValues(rv);
    }

    public boolean getTransaction_RestoreValues() {
		return pmf.getRestoreValues();
    }
    public void setTransaction_RestoreValues(boolean rv) {
		pmf.setRestoreValues(rv);
    }
    
    public boolean getTransaction_MultiThread() {
		return pmf.getMultithreaded();
    }
    public void setTransaction_MultiThread(boolean mt) {
		pmf.setMultithreaded(mt);
    }

    public boolean getIsOpen() {
        return !pmf.isClosed();
    }

    public String getMappingStructure() {
		return AbstractSpeedo.getMappingStructureString(jf.getMappingStructureRule());
    }

    public boolean getDebug() {
		return Boolean.valueOf(
		        pmf.getProperties().getProperty(SpeedoProperties.DEBUG))
		        .booleanValue();
    }

    public void setDebug(boolean d) {
        pmf.getProperties().setProperty(
                SpeedoProperties.DEBUG, Boolean.TRUE.toString());
    }

    public Collection getPersistentClasseNames() {
        return Arrays.asList(mapper.getMappedClasses());
    }

    public void clean() {
    	try {
    		pmf.clean();
    	} catch (Exception e) {
			e.printStackTrace();
		}
    }

    public Map getProperties() {
        return jfa.getSpeedoProperties();
    }

    public void setProperty(String name, String value) {
        jfa.setSpeedoProperty(new String[]{name, value});
    }
    
}
