/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.mapper.api;

import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.naming.api.PBinder;

/**
 * It defines a Jorm configurator which manages the PClassMapping, the PBinder
 * and the PNamingContext for persistent classes.
 *
 * @author S.Chassande-Barrioz
 */
public interface JormFactory {

    /**
     * Retrieve the Binder managing the persistent class (Not a Generic class).
	 * This method is a shortcut to this call:
	 *   getPClassMapping(clazz).getPBinder()
	 *
     * @param clazz the Jorm class name managed which the PBinder is asked
     * @return the PBinder instance to use for the given jorm class name
     */
    PBinder getPBinder(Class clazz) throws PException;

	/**
	 * Retrieve the Binder managing the persistent class (Not a Generic class).
	 * This method is a shortcut to this call:
	 *   getPClassMapping(Class.forName(classname, cl, true)).getPBinder()
	 *
	 * @param classname is the class name of the jorm class which the
	 * PBinder is required
	 * @param cl is the classloader where to find/load the class
	 * @return the PBinder instance to use for the given jorm class name
	 */
    PBinder getPBinder(String classname, ClassLoader cl) throws PException;

    /**
     * Is a shortcut to the call
	 * getPClassMapping(Class.forName(classname, cl, true))
	 *
     * @param classname is the class name of the jorm class which the
     * PClassMapping is required
	 * @param cl is the classloader where to find/load the class
     * @return the PClassMapping which manages a PersistentObjectItf
     */
    PClassMapping getPClassMapping(String classname, ClassLoader cl) throws PException;

	/**
	 * Retrieves a PClassMapping instance managing a jorm class. If the
	 * PClassMapping does not already exist then it is created, configured and
	 * mapped in the PMapper. The PClassMapping configuration contains the
	 * PBinder assignement, the PNamingContext assignement for each reference,
	 * and the GenClassMapping assignement. The assigned GenClassMapping must
	 * also be configured. In other hand the PClassMapping of referenced class
	 * is not configured now. Only the PNamingContext is needed.
	 * @param clazz is the class of the jorm class which the
	 * PClassMapping is required
	 * @return the PClassMapping which manages the persistent class
	 */
    PClassMapping getPClassMapping(Class clazz) throws PException;

	/**
	 * Retrieve the PClassMapping of a gen class since its path
	 * @param path is the path representing the way to access the
	 * GenClassMapping
	 */
    PClassMapping getGenClassMapping(String path);

    /**
	 * Is a shortcut to the call
	 * getPNamingContext(Class.forName(classname, cl, true))
	 *
	 * @see #getPNamingContext(Class)
     * @param classname the Jorm class name managed by the wanted PNamingContext
     * @return the PNamingContext instance to use for the given jorm class name
     */
    PNamingContext getPNamingContext(String classname, ClassLoader cl) throws PException;

	/**
	 * Retrieve the PNamingcontext managing the persistent class (Not a
	 * Generic class) and its sub classes.
	 *
	 * @param clazz the Jorm class name managed which the PNamingContext is asked
	 * @return the PBinder instance to use for the given jorm class name
	 */
    PNamingContext getPNamingContext(Class clazz) throws PException;

    /**
     * Retrieves the classloader of a class if the class has been already mapped.
     * Otherwise a null value is returned
     */ 
    ClassLoader getClassLoader(String className);

	/**
	 * @see org.objectweb.speedo.mapper.api.JormFactoryAttributes
	 */
	byte getMappingStructureRule();
	
	/**
	 * Clean the jorm factory forgetting all application classes loaded previously.
	 */
	void clean() throws PException;

}
