/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.mapper.lib;

import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.lib.BooleanHelper;
import org.objectweb.speedo.mapper.api.ClassPropertyManager;
import org.objectweb.speedo.mim.api.HomeItf;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.Properties;
import java.util.StringTokenizer;


public class CPMUserCachePolicy implements ClassPropertyManager, SpeedoProperties {

	public String getName() {
		return USER_CACHE_CLASS_POLICY;
	}

	public void applyDefault(HomeItf home, Properties props, Logger l) {
	}
	public void applyDefault(HomeItf gcHome, HomeItf refHome, Properties props, Logger l) {
	}
	public void applyProperty(String name, String value, 
			HomeItf home, Properties props, Logger logger) {
        StringTokenizer st = new StringTokenizer(value, " ,;:/\t", false);
        while(st.hasMoreTokens()) {
            String cacheName = st.nextToken();
            home.activeUserCache(cacheName);
	        if (logger.isLoggable(BasicLevel.INFO)) {
	            logger.log(BasicLevel.INFO, "Activating the user cache '" + cacheName 
	                    + "' for the class '" + home.getPath());
	        }
	        //activate cache for all subclasses
	        PClassMapping[] subPcms = null;
	        try {
	        	subPcms = home.getSubPCMs();
	        } catch (Exception e) {
				logger.log(BasicLevel.WARN, "cannot apply property '" 
						+ name + "' to the sub classes of '" 
						+ home.getPath() + "': ", e);
	        }
        	if (subPcms != null) {
        		for(int i=0; i<subPcms.length; i++) {
        			((HomeItf)subPcms[i]).activeUserCache(cacheName);
        		}
        	}
        }
	}
	public void applyProperty(String name, String value, 
			HomeItf gcHome, HomeItf refHome, Properties props, Logger logger) {
	}
}
