/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 * 
 */
package org.objectweb.speedo.metadata;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;


/**
 * @author Y.Bersihand
 */
public class SpeedoFetchGroup extends SpeedoElement {
	
	public static final String FG_AT = "@";
	public static final String FG_SHARP = "#";
	public static final String FG_DOT = ".";
	public static final String FG_KEY = "#key";
	public static final String FG_VALUE = "#value";
	public static final String FG_ELEMENT = "#element";
	public static final String FG_SLASH = "/";
	
	public static final byte NOTHING_DEFINED = 0;
	public static final byte DEPTH_DEFINED = 1;
	public static final byte FG_DEFINED = 2;
	
	/**
	 * The name of the fetch group.
	 */
	public String name;
	
	/**
	 * The list of nested fetchgroups. 
	 */
	protected  Map nestedFetchGroups = new HashMap();

	/**
	 * The list of fields declared to be in this fetch group.
	 */
	protected Map fields = new HashMap();

	/**
	 * Recursive fetch group references are controlled by the depth attribute. A
	 * depth of 0 (the default) will fetch the whole graph of instances
	 * reachable from this field.
	 */
	public int depth = 0;

	/**
	 * The post-load attribute on the fetch-group element indicates whether the
	 * jdoPost-Load callback will be made when the fetch group is loaded. It
	 * defaults to false, for all fetch groups except the default fetch group,
	 * on which it defaults to true.
	 */
	public boolean postLoad;
 
	/**
	 * @return Return the depth.
	 */
	public int getDepth(){
	   return depth;
	}

	/**
	 * @return Returns the name of the fetchgroup
	 */
	public String getName(){
		return name;
	}

	/**
	 * @return Returns the postLoad.
	 */
	public boolean getPostLoad() {
		return postLoad;
	}

	/**
	 * @return Return the map of fields.
	 */
	public Map getFields(){
		return fields;
	}
	
	/**
	 * @return Returns the map of nested fetch groups
	 */
	public Map getNestedFetchGroups() {
		return nestedFetchGroups;
	}

	/**
	 * Add a new FetchGroupMetaData
	 * @param fetchGroup is the group to add
	 */
	public void addFetchGroup(Object fetchGroup) {
		SpeedoFetchGroup speedoFetchGroup = (SpeedoFetchGroup) fetchGroup;
		nestedFetchGroups.put(speedoFetchGroup.name, speedoFetchGroup);
	}

	/**
	 * Add a new SpeedoField
	 * @param field is the field to add
	 */
	public void addField(Object field) {
		SpeedoField speedoField = (SpeedoField) field;
		fields.put(speedoField.name, speedoField);
	}
	
	/**
	 * @return the set of the names of the fields to load with this fetchgroup
	 */
	public Set getFieldsToLoad(){
		Set fieldsToLoad = new HashSet();
		//add all the fields present in the jdoFields
		fieldsToLoad.addAll(fields.keySet());
		//recursive call on all the nested fetch groups
		Set fg = nestedFetchGroups.entrySet();
		Iterator it = fg.iterator();
		while(it.hasNext()){
			SpeedoFetchGroup sfg = (SpeedoFetchGroup) it.next();
			fieldsToLoad.addAll(sfg.fields.keySet());
		}
		return fieldsToLoad;	
	}
	
	public String toString(){
		String s = "\n fetchgroup name: " + name +
					", postload: " + postLoad +
					", depth: " + depth;
		s += ", \t fields:[";
		Iterator it = fields.values().iterator();
		while (it.hasNext()) {
			s = s + "\t" + it.next().toString();
		}
		s += "], \t nestedFetchGroups:[";
		Iterator it2 = nestedFetchGroups.values().iterator();
		while (it2.hasNext()) {
			s += "\t" + it2.next().toString();
		}
		s += "]";
		return s;
	}
}
