/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.metadata;

/**
 * This class corresponds to the description of the attribute version
 * in the XML file.
 * @author Y.Bersihand
 */
public class SpeedoVersion {
	
	public final static byte VERSION_NUMBER = 3;
    public final static byte STATE_COMPARISON = 2;
    /**
     * Versioning strategy based on the date
     */
    public final static byte DATE_TIME = 1;
    public final static byte NO_VERSION = 0;
    
	/**
	 * The strategy followed 
	 */
	public byte strategy = NO_VERSION;
	
    /**
     * Transforms a String into a Byte. The String must corresponds to local variables.
     * It returns the byte associated with the variable.
     * @param s String to transform.
     * @return the byte associated to the String.
     */
    public static byte toByte(String s) {
        if (s.equalsIgnoreCase("version-number"))
            return VERSION_NUMBER;
        else if (s.equalsIgnoreCase("state-comparison"))
            return STATE_COMPARISON;
		else if (s.equalsIgnoreCase("date-time"))
				   return DATE_TIME; 
        else
            return NO_VERSION;
    }
    
	/**
	 * Transforms a byte into a String. 
	 * @param b the byte to transform.
	 * @return the String associated to the byte.
	 */
	public static String toString(byte b) {
	   if (b == VERSION_NUMBER)
		   return "versionnumber";
	   else if (b == STATE_COMPARISON)
		   return "statecomparison";
	   else if (b == DATE_TIME)
				  return  "datetime"; 
	   else
		   return "no-version";
	}
} 
