/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.mim.lib;

import org.objectweb.speedo.mim.api.LifeCycle;
import org.objectweb.speedo.mim.api.DetachedLifeCycle;
import org.objectweb.speedo.mim.api.StateItf;
import org.objectweb.speedo.mim.api.PersistentObjectItf;
import org.objectweb.perseus.cache.api.CacheEntry;

/**
 * This class is the basic implementation of the StateItf interface. It
 * is used at the top of the XXXFields class inheritance.
 *
 * @author S.Chassande-Barrioz
 */
public abstract class AbstractStateImpl implements StateItf {

	/**
	 * This version is allocated:
	 * 	- before attach
	 * 	- before detach
	 * 	- before commit
	 */
	
    protected byte jdoStatus = LifeCycle.initState();

	public byte detachedStatus = DetachedLifeCycle.initState();
	
	protected boolean isToMerge = false;
	
    public PersistentObjectItf speedoPO = null;

	public boolean hasBeenFlush = false;

	public AbstractStateImpl() {
	}

	public AbstractStateImpl(PersistentObjectItf thepo) {
		this.speedoPO = thepo;
	}

	// IMPLEMENTATION OF THE StateItf INTERFACE //
	//---------------------------------------//

	public CacheEntry getCacheEntry() {
		return speedoPO;
	}
	public void setCacheEntry(CacheEntry ce) {
		speedoPO = (PersistentObjectItf) ce;
	}

	
    // IMPLEMENTATION OF THE PAccessor INTERFACE //
    //-------------------------------------------//

    public Object getMemoryInstance() {
        return speedoPO;
    }


    // IMPLEMENTATION OF THE StateItf INTERFACE //
    //------------------------------------------------//

    /**
     * Changes the status of this object
     *
     * @see org.objectweb.speedo.mim.api.LifeCycle
     * @param action the action that may change the status
     */
    public void speedoChangeStatus(byte action) {
		byte neo = LifeCycle.nextStatePersistenceCapable(jdoStatus, action);
		if (neo == LifeCycle.ERROR) {
			throw new RuntimeException("Operation "
					+ LifeCycle.actionToString(action)
					+ " not allowed with the status "
					+ LifeCycle.statusToString(jdoStatus));
		}
        jdoStatus = neo;
    }

    /**
     * Gets the current status of this object
     *
     * @see org.objectweb.speedo.mim.api.LifeCycle
     * @return the current state in the life cycle
     */
    public byte speedoGetStatus() {
        return jdoStatus;
    }

    /**
     * Forces the new value of the status
     *
     * @see org.objectweb.speedo.mim.api.LifeCycle
     * @param newValue the new status of this object
     */
    public void speedoSetStatus(byte newValue) {
        jdoStatus = newValue;
    }

	public byte getDetachedStatus() {
		return detachedStatus;
	}
	
	public void setDetachedStatus(byte newValue) {
		detachedStatus = newValue;
	}
	
    /**
     * @return The PersistentObjectItf attached to this state representation.
     */
    public PersistentObjectItf getSpeedoPO() {
        return speedoPO;
    }

    /**
     * It assignes the PersistentObjectItf attached to this state representation.
     */
    public void setSpeedoPO(PersistentObjectItf sp) {
        speedoPO = sp;
    }

    public void prepareWrite() {
        // do nothing
    }

	public void workingSetClosed() {
		// do nothing
	}

	public boolean hasBeenFlush() {
		return hasBeenFlush;
	}

	public void setFlushed(boolean val) {
		hasBeenFlush = val;
	}

	public void speedoChangeVersion() {
		//do nothing
	}

	public boolean checkVersion(StateItf sa) {
		return true;
	}
	
	public long getVersion() {
		return 0;
	}
	
    public boolean isToMerge() {
        return this.isToMerge;
    }
    public void makeToMerge(Object thinLock) {
        this.isToMerge = true;

    }
    public org.objectweb.perseus.persistence.api.State merge(org.objectweb.perseus.persistence.api.State oldState) {
        this.isToMerge = false;
        return oldState;
    }
    
    public Object getUserKey(int cacheId) {
        return null;
    }
    
    public void indexFieldModified(int cacheId, boolean rebind) {
    }
}
