/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.pm.api;

import java.util.Collection;


/**
 * This interfaces defines a POManagerItf switch. It binds a thread to a
 * list of POManagerItf. In a same context two po managers cannot be managed
 * by the same PersistentManagerFactory.
 *
 * @author S.Chassande-Barrioz
 */
public interface POManagerSwitchItf {

    /**
     * @param pmf is persistent manager factory which manages the returned
     * po manager.
     * @return the POManagerItf managed by the given persistence manager factory
     * and bound to current the context, or the null value if there is no
     * POManagerItf.
     */
    POManagerItf lookup(POManagerFactoryItf pmf);

    /**
     * It assignes a POManagerItf to the current context.
     * @param pm is the POManagerItf
     */
    void bind(POManagerItf pm);

    /**
     * It clears the list of POManagerItf for the current context.
     */
    void clear();

    /**
     * It clears a POManagerItf for the current context.
	 * @return true if the specified pm has been unbound from the context/thread
     */
    boolean unbind(POManagerItf pm);

	/**
	 * It clears the POManagerItf associated to a POManagerFactoryItf for the
	 * current context
	 * @return true if a pm associated to the specified PMF, has been unbound
	 * from the context/thread
	 */
    boolean unbind(POManagerFactoryItf pmf);

    /**
     * @return all POManagerItf instances bound with the current context
     */
    Collection entries();
}
