/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S. Chassande-Barrioz
 *
 */
package org.objectweb.speedo.pm.jdo.lib;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Savepoint;
import java.sql.Statement;
import java.util.Map;

import javax.jdo.JDOUserException;

/**
 * This class implements the java.sql.Connection and delegates call to an inner
 * connection. But some calls are forbidden by the JDO 2 specification (12.16).
 * 
 * @author S.Chassande-Barrioz
 */
public class DSConnectionFilter implements Connection {
    
    Connection inner;
    
    /**
     * indicates if the inner connection must be closed or simply forgotten.
     */
    boolean reallyClose;
    
    /**
     * @param inner
     * @param reallyClose
     */
    public DSConnectionFilter(Object inner, boolean reallyClose) {
        super();
        this.inner = (Connection) inner;
        this.reallyClose = reallyClose;
    }
    public void clearWarnings() throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public void close() throws SQLException {
        if (reallyClose) {
            inner.close();
        }
        inner = null;
    }
    public void commit() throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public Statement createStatement() throws SQLException {
        return inner.createStatement();
    }
    public Statement createStatement(int resultSetType,
            int resultSetConcurrency, int resultSetHoldability)
            throws SQLException {
        return inner.createStatement(
                resultSetType,resultSetConcurrency, resultSetHoldability);
    }
    public Statement createStatement(int resultSetType, int resultSetConcurrency)
            throws SQLException {
        return inner.createStatement(resultSetType, resultSetConcurrency);
    }
    public boolean getAutoCommit() throws SQLException {
        return inner.getAutoCommit();
    }
    public String getCatalog() throws SQLException {
        return inner.getCatalog();
    }
    public int getHoldability() throws SQLException {
        return inner.getHoldability();
    }
    public DatabaseMetaData getMetaData() throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public int getTransactionIsolation() throws SQLException {
        return inner.getTransactionIsolation();
    }
    public Map getTypeMap() throws SQLException {
        return inner.getTypeMap();
    }
    public SQLWarning getWarnings() throws SQLException {
        return inner.getWarnings();
    }
    public boolean isClosed() throws SQLException {
        return inner.isClosed();
    }
    public boolean isReadOnly() throws SQLException {
        return inner.isReadOnly();
    }
    public String nativeSQL(String sql) throws SQLException {
        return inner.nativeSQL(sql);
    }
    public CallableStatement prepareCall(String sql, int resultSetType,
            int resultSetConcurrency, int resultSetHoldability)
            throws SQLException {
        return inner.prepareCall(sql, 
                resultSetType, resultSetConcurrency, resultSetHoldability);
    }
    public CallableStatement prepareCall(String sql, int resultSetType,
            int resultSetConcurrency) throws SQLException {
        return inner.prepareCall(sql, resultSetType, resultSetConcurrency);
    }
    public CallableStatement prepareCall(String sql) throws SQLException {
        return inner.prepareCall(sql);
    }
    public PreparedStatement prepareStatement(String sql, int resultSetType,
            int resultSetConcurrency, int resultSetHoldability)
            throws SQLException {
        return inner.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
    }
    public PreparedStatement prepareStatement(String sql, int resultSetType,
            int resultSetConcurrency) throws SQLException {
        return inner.prepareStatement(sql, resultSetType, resultSetConcurrency);
    }
    public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys)
            throws SQLException {
        return inner.prepareStatement(sql, autoGeneratedKeys);
    }
    public PreparedStatement prepareStatement(String sql, int[] columnIndexes)
            throws SQLException {
        return inner.prepareStatement(sql, columnIndexes);
    }
    public PreparedStatement prepareStatement(String sql, String[] columnNames)
            throws SQLException {
        return inner.prepareStatement(sql, columnNames);
    }
    public PreparedStatement prepareStatement(String sql) throws SQLException {
        return inner.prepareStatement(sql);
    }
    public void releaseSavepoint(Savepoint savepoint) throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public void rollback() throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public void rollback(Savepoint savepoint) throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public void setAutoCommit(boolean autoCommit) throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public void setCatalog(String catalog) throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public void setHoldability(int holdability) throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public void setReadOnly(boolean readOnly) throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public Savepoint setSavepoint() throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public Savepoint setSavepoint(String name) throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public void setTransactionIsolation(int level) throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
    public void setTypeMap(Map map) throws SQLException {
        throw new JDOUserException("Action forbidden on JDO connection");
    }
}
