/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.sequence.api;

import javax.jdo.datastore.Sequence;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.speedo.pm.api.POManagerFactoryItf;

/**
 * Class to manage sequences defined for a pmf.
 * @author Y.Bersihand
 */
public interface SequenceManager {

	/**
	 * Get the long generator manager.
	 */
	Object getLongGenMgr ();
	
	/**
	 * Set the long generator manager.
	 */
	void setLongGenMgr (Object longGenMgr);
	
	/**
     * Add the sequence if not already registered in the list of sequences defined for this pmf.
     * @param sequence
     */
    void addSequence(Object sequence); 
    
    /**
     * Try to remove the sequence identified by the sequenceName 
     * from the list of sequences defined for this pmf.
     * If the sequence does not exist, nothing is done.
     * @param sequenceName the fully qualified name of the sequence, 
     * i.e "packageName.sequenceName".
     */
    void removeSequence(String sequenceName);
    
    /**
     * Return the sequence associated to the name parameter.
     * If no sequence is found, return null.
     * @param name the fully qualified name of the sequence, 
     * i.e "packageName.sequenceName".
     */
    SpeedoSequenceItf getSequence(String name);
    
    PMapper getMapper();

	void setMapper(PMapper mapper);
	
	POManagerFactoryItf getPMF();
	
	void setPMF(POManagerFactoryItf pmf);
}
