/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.speedo.workingset.api;

import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.PersistenceException;
import org.objectweb.perseus.persistence.api.TransactionalWorkingSet;
import org.objectweb.speedo.pm.api.POManagerItf;

/**
 * Defines the transaction concept. It offers 
 * - demarcation methods (begin, commit rollback)
 * - status checking
 * - management of the connection to the underlying data support 
 *
 * @author S.Chassande-Barrioz
 */
public interface TransactionItf extends TransactionalWorkingSet {

    /**
     * @return boolean value indicating if the transaction is active or not.
     */
    boolean isActive();
    
    /**
     * Starts the transaction.
     * @throws SpeedoRuntimeException If the transaction is already started 
     */
    void begin();
    
    /**
     * Commits the transaction.
     * @throws SpeedoRuntimeException If the commit process fails 
     */
    void commit();
    
    /**
     * Rolles back the transaction.
     * @throws SpeedoRuntimeException If the rollback process fails 
     */
    void rollback();
    
    
    /**
     * It activates the working set. This is used to delimit the begining of
     * the working set.
     */
    void activate() throws PersistenceException;

    /**
     * Indicates if the transaction environnement is managed. If the 'false'
     * value is returned then that means that a user transaction matches to
     * a local transaction managed by speedo.
     */ 
    boolean isManagedEnv();

    /**
     * Assignes the connection holder. The connection holder is able to allocate
     * a connection to the underlying data support and to kept it until the
     * working set / transaction end.
     */
    void setConnectionHolder(ConnectionHolder ch);

    /**
     * Marks the current transaction as rollback only. this means that the 
     * transaction cannot be committed
     */
    void setRollbackOnly();

    /**
     * @return the rollback only flag.
     */
    boolean getRollbackOnly();
    
    RuntimeException rollBackOnInternalError(Exception e);
    
    POManagerItf getPOManager();
}
