/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.speedo.workingset.ejb.lib;

import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.api.SpeedoRuntimeException;
import org.objectweb.speedo.pm.jdo.api.JDOPOManagerItf;
import org.objectweb.speedo.workingset.ejb.api.EJBTransactionItf;
import org.objectweb.speedo.workingset.lib.AbstractTransaction;
import org.objectweb.util.monolog.api.BasicLevel;

import javax.persistence.PersistenceException;
import javax.naming.InitialContext;
import javax.transaction.TransactionManager;

/**
 * Implements the EJB transaction API since the generic speedo implementation.
 *
 * @author S.Chassande-Barrioz
 */
public class EJBTransactionImpl extends AbstractTransaction 
    implements EJBTransactionItf {

    public EJBTransactionImpl() {
        super();
    }

    public void beforeWSPrepare() throws org.objectweb.perseus.persistence.api.PersistenceException {
        try {
            super.beforeWSPrepare();
        } catch(SpeedoRuntimeException e) {
            Exception ne;
            if (e.getCause() != null) {
                ne = new PersistenceException(e.getMessage(), e.getCause());
            } else {
                ne = new PersistenceException(e.getMessage());
            }
            throw new org.objectweb.perseus.persistence.api.PersistenceException(ne);
        }
    }
    
    public void onWSEnd() {
        try {
            super.onWSEnd();
        } catch(SpeedoRuntimeException e) {
            if (e.getCause() != null) {
                throw new PersistenceException(e.getMessage(), e.getCause());
            } else {
                throw new PersistenceException(e.getMessage());
            }
        }
    }
    
    public void begin() {
        logger.log(BasicLevel.INFO, "Begin the transaction");
        try {
            super.begin();
        } catch (SpeedoRuntimeException e) {
            throw new PersistenceException(e.getMessage(), e.getCause());
        }
    }

    public void commit() {
        try {
            super.commit();
        } catch (SpeedoRuntimeException e) {
            throw new PersistenceException(e.getMessage(), e);
        }
    }

    public void rollback() {
        try {
            super.commit();
        } catch (SpeedoRuntimeException e) {
            throw new PersistenceException(e.getMessage(), e);
        }
    }
    public RuntimeException rollBackOnInternalError(Exception _e) {
        Exception ie = ExceptionHelper.getNested(_e);
        if (isActive()) {
            if (managedEnv) {
                logger.log(BasicLevel.INFO, ".");
                String tmName = ((JDOPOManagerItf) pm).getPersistenceManagerFactory().getProperties()
                        .getProperty(SpeedoProperties.TM_NAME);
                if (tmName == null) {
                    return new PersistenceException("No transaction manager jndi name found in initialisation properties");
                }
                try {
                    Object o = new InitialContext().lookup(tmName);
                    if (o == null) {
                        String msg = "The transaction must be marked as rollbackOnly: JNDI retrieves a null transaction manager for the name '" + tmName + "'.";
                        logger.log(BasicLevel.ERROR, msg);
                        return new PersistenceException(msg);
                    }
                    if (!(o instanceof TransactionManager)) {
                        String msg = "The transaction must be marked as rollbackOnly: JNDI retrieves an object which is not a javax.transaction.TransactionManager (JNDI name: " + tmName + "): " + o;
                        logger.log(BasicLevel.ERROR, msg);
                        return new PersistenceException(msg);
                    }
                    javax.transaction.Transaction t = ((TransactionManager) o).getTransaction();
                    if (t == null) {
                        String msg = "Impossible to rollback an unexisting transaction (TM.getTransaction()=null)";
                        logger.log(BasicLevel.ERROR, msg);
                        return new PersistenceException(msg);
                    }
                    t.setRollbackOnly();
                } catch (Exception e) {
                    String msg = "The transaction must be marked as rollbackOnly: Error when lookup the transaction manager in JNDI with the name '"
                            + tmName + "'";
                    logger.log(BasicLevel.ERROR, msg, e);
                    return new PersistenceException(msg);
                }
            } else {
                rollback();
            }
            return new PersistenceException("The current transaction has been rolledback, please retry", ie);
        } else {
            return new PersistenceException("The current working set occrurs an error, please retry", ie);
        }
    }

}
