/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.ejb.pobjects.basic;

import junit.framework.Assert;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.PrimaryKeyJoinColumn;
import javax.persistence.SecondaryTable;
import javax.persistence.SecondaryTables;
import javax.persistence.Table;

/**
 * This class defines a mapping to two tables: a main table and several
 * secondary tables.
 * @author P. Dechamboux
 */
@Entity
@Table(name="EJB_SEVSECONDARYM")
@SecondaryTables({
	@SecondaryTable(
			name="EJB_SEVSECONDARYS1",
			pkJoinColumns=@PrimaryKeyJoinColumn(
					name="S1ID",
					referencedColumnName="ID"
				)
		),
	@SecondaryTable(
			name="EJB_SEVSECONDARYS2",
			pkJoinColumns=@PrimaryKeyJoinColumn(
					name="S2ID",
					referencedColumnName="ID"
				)
		)}
)
public class SeveralSecondary implements Serializable {
	@Id
	@Column(name="ID")
	private Long id;
	@Column(name="MAIN")
	private int int_main;
	// TODO: Does not support same column name into two different table: coorect it!!
	@Column(name="SECONDARY1",table="EJB_SEVSECONDARYS1")
	private int int_secondary1;
	@Column(name="SECONDARY2",table="EJB_SEVSECONDARYS2")
	private int int_secondary2;
	
	public void assign(long longval) {
		this.int_main = (int) longval;
		this.int_secondary1 = (int) longval + 1;
		this.int_secondary2 = (int) longval + 2;
	}

	public void check(long longval) {
		Assert.assertEquals("bad int_main value", int_main, (int) longval);
		Assert.assertEquals("bad int_secondary value", int_secondary1, (int) longval + 1);
		Assert.assertEquals("bad int_secondary value", int_secondary2, (int) longval + 2);
	}

	public int retrieve_int_field() {
		return int_main;
	}

	public void assign_int_field(int int_field) {
		this.int_main = int_field;
		this.int_secondary1 = int_field + 1;
		this.int_secondary2 = int_field + 2;
	}
}
