/*
 * Created on 16 juin 2005
 *

 */
package abook.data;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;


/**
 * Provides object of type Book mapping the data stored in the sql database
 */
public class Book implements Serializable{

	private long bookId;
	private String longTitle;
	private String shortTitle;
	private double totalDuration; // seconds
	private String releaseDate;
	private String imageFilePath;
	private String urlBook;
	private String urlSummary;
	private String publisher;
	private String bookKind;
	private String bookType;
	private int popularity;
	private	Collection authors;
	private Collection narrators;
	private String mainAuthor;
	private String mainNarrator;
	
	/**
	 * Constructs a new Book object
	 * @param bookId The book identifier.
	 * @param title The book title.
	 */
	public Book(String title, long bookId) {
		this.longTitle = title;
		if (title.length()<15){
			this.shortTitle = title;
		}
		else{
			this.shortTitle = (title.substring(0,12)+ "...");
		}
		this.bookId = bookId;		
		authors = new ArrayList();
		narrators = new ArrayList();
	}
	/**
	 * @return Returns the authors.
	 */
	public Collection getAuthors() {
		return authors;
	}
	/**
	 * @param authors The authors to set.
	 */
	public void setAuthors(Collection authors) {
		this.authors = authors;
	}
	/**
	 * @return Returns the bookId.
	 */
	public long getBookId() {
		return bookId;
	}
	/**
	 * @param bookId The bookId to set.
	 */
	public void setBookId(long bookId) {
		this.bookId = bookId;
	}
	/**
	 * @return Returns the imageFilePath.
	 */
	public String getImageFilePath() {
		return imageFilePath;
	}
	/**
	 * @param imageFilePath The imageFilePath to set.
	 */
	public void setImageFilePath(String imageFilePath) {
		this.imageFilePath = imageFilePath;
	}
	/**
	 * @return Returns the longTitle.
	 */
	public String getLongTitle() {
		return longTitle;
	}
	/**
	 * @param longTitle The longTitle to set.
	 */
	public void setLongTitle(String longTitle) {
		this.longTitle = longTitle;
	}
	/**
	 * @return Returns the narrators.
	 */
	public Collection getNarrators() {
		return narrators;
	}
	/**
	 * @param narrators The narrators to set.
	 */
	public void setNarrators(Collection narrators) {
		this.narrators = narrators;
	}
	/**
	 * @return Returns the releaseDate.
	 */
	public String getReleaseDate() {
		return releaseDate;
	}
	/**
	 * @param releaseDate The releaseDate to set.
	 */
	public void setReleaseDate(String releaseDate) {
		this.releaseDate = releaseDate;
	}
	/**
	 * @return Returns the shortTitle.
	 */
	public String getShortTitle() {
		return shortTitle;
	}
	/**
	 * @param shortTitle The shortTitle to set.
	 */
	public void setShortTitle(String shortTitle) {
		this.shortTitle = shortTitle;
	}
	/**
	 * @return Returns the totalDuration.
	 */
	public double getTotalDuration() {
		return totalDuration;
	}
	/**
	 * @param totalDuration The totalDuration to set.
	 */
	public void setTotalDuration(double totalDuration) {
		this.totalDuration = totalDuration;
	}
	
	/**
	 * @return Returns the bookType.
	 */
	public String getBookType() {
		return bookType;
	}
	/**
	 * @param bookType The bookType to set.
	 */
	public void setBookType(String bookType) {
		this.bookType = bookType;
	}
	/**
	 * @return Returns the urlBook.
	 */
	public String getUrlBook() {
		return urlBook;
	}
	/**
	 * @param urlBook The urlBook to set.
	 */
	public void setUrlBook(String urlBook) {
		this.urlBook = urlBook;
	}
	/**
	 * @return Returns the urlSummary.
	 */
	public String getUrlSummary() {
		return urlSummary;
	}
	/**
	 * @param urlSummary The urlSummary to set.
	 */
	public void setUrlSummary(String urlSummary) {
		this.urlSummary = urlSummary;
	}
	
	
	/**
	 * @return Returns the bookKind.
	 */
	public String getBookKind() {
		return bookKind;
	}
	/**
	 * @param bookKind The bookKind to set.
	 */
	public void setBookKind(String bookKind) {
		this.bookKind = bookKind;
	}
	/**
	 * @return Returns the popularity.
	 */
	public int getPopularity() {
		return popularity;
	}
	/**
	 * @param popularity The popularity to set.
	 */
	public void setPopularity(int popularity) {
		this.popularity = popularity;
	}
	/**
	 * @return Returns the publisher.
	 */
	public String getPublisher() {
		return publisher;
	}
	/**
	 * @param publisher The publisher to set.
	 */
	public void setPublisher(String publisher) {
		this.publisher = publisher;
	}
	
	
	/**
	 * @return Returns the mainAuthor.
	 */
	String getMainAuthor() {
		return mainAuthor;
	}
	/**
	 * @param mainAuthor The mainAuthor to set.
	 */
	void setMainAuthor(String mainAuthor) {
		this.mainAuthor = mainAuthor;
	}
	/**
	 * @return Returns the mainNarrator.
	 */
	String getMainNarrator() {
		return mainNarrator;
	}
	/**
	 * @param mainNarrator The mainNarrator to set.
	 */
	void setMainNarrator(String mainNarrator) {
		this.mainNarrator = mainNarrator;
	}
	/**
	 * @param author an author to add to the authors collection of the book
	 * @return true if this collection changed as a result of the call
	 */
	public boolean add(Object author) {
		return authors.add(author);
	}
	/**
	 * @param arg0 a collection of authors
	 * @return if this collection changed as a result of the call
	 */
	public boolean addAll(Collection arg0) {
		return authors.addAll(arg0);
	}
	/**
	 * @param author The author to add.
	 */
	public void addAuthor(Author author){
		authors.add(author);
		author.addBook(this);
	}
	/**
	 * @param narrator The narrator to add.
	 */
	public void addNarrator(Narrator narrator){
		narrators.add(narrator);
		narrator.addBook(this);
	}
	/**
	 * @return Returns the totalDuration in a String format.
	 */
	public String getStrTotalDuration() {
		if (totalDuration == 0){
			return "";
		}
		long h = (long)totalDuration / 3600;
		long m = ((long)totalDuration - (h * 3600)) / 60;
		long s = ((long)totalDuration - (m * 60));
		String result = String.valueOf(h) + ":" + String.valueOf(m) + ":" + String.valueOf(s);
		return result;
	}
}














