/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.pobjects.fetchgroup;

import java.util.Iterator;
import java.util.Set;

/**
 * @author Y.Bersihand
 */
public class Person {

	private String name;
	private Address address;
	private Set children;
	private int age;
	
	public Person() {
	}
	
	public Person(String name, Address address, Set children, int age){
		this.name = name;
		this.address = address;
		this.children = children;
		this.age = age;
	}
	
	public Address getAddress() {
		return address;
	}
	
	public void setAddress(Address address) {
		this.address = address;
	}
	
	public Set getChildren() {
		return children;
	}
	
	public void setChildren(Set children) {
		this.children = children;
	}
	
	public boolean hasChildren(){
		if(children == null)
			return false;
		return !children.isEmpty();
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public int getAge() {
		return age;
	}
	
	public void setAge(int age) {
		this.age = age;
	}
	
	public String toString(){
		String s =  "Person[" + name + ", " + age;
		if(address != null)
			s += ", " + address.toString();
		else
			s += ", no address";
		if(hasChildren()){
			s += "\n";
			Iterator it = children.iterator();
			while(it.hasNext()){
				Person p = (Person) it.next();
				s += "\t" + p.toString() + "\n";
			}
		}
		else{
			s += ", no child";
		}
		s += "]";
		return s;
	}
}
