/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.runtime.basic;

import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.jorm.mapper.rdb.lib.ConnectionSpecJDBC;

import javax.naming.InitialContext;
import javax.sql.DataSource;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

import junit.framework.Assert;

/**
 * @author S.Chassande-Barrioz
 */
public class TestJavaxSqlDatasource extends TestBasicA {

    static {
        System.setProperty("java.naming.factory.initial",
            "org.objectweb.speedo.runtime.basic.SpeedoInitialNamingFactory");
    }

    public final static String CF_NAME = "ds";
    public TestJavaxSqlDatasource(String s) {
        super(s);
    }


    public Properties getPMFProperties() {
		Properties p = super.getPMFProperties();
		p.setProperty("javax.jdo.option.ConnectionFactoryName", CF_NAME);
		if (pmf == null) {
			String dcn = getConnectionDriverNameProperty(p);
			String user = p.getProperty(SpeedoProperties.JDO_OPTION_CONNECTION_USER_NAME);
			String pass = p.getProperty(SpeedoProperties.JDO_OPTION_CONNECTION_PASSWORD, "");
			String url = p.getProperty(SpeedoProperties.JDO_OPTION_CONNECTION_URL);
			if (user == null) {
				System.out.println(p);
				fail("No user name specified");
			}
			if (url == null) {
				System.out.println(p);
				fail("No database url specified");
			}
			try {
				DriverManager.registerDriver((Driver)
							Class.forName(dcn).newInstance());
				Object cf = new ConnectionSpecJDBC(url, dcn, user, pass);
				InitialContext ic = new InitialContext();
				ic.rebind(CF_NAME, cf);
			} catch (Exception e) {
				e.printStackTrace();
				fail(e.getMessage());
			}
		}
        return p;
    }

    public void testGetSqlConnection() {
        logger.log(BasicLevel.DEBUG, "testGetSqlConnection");
        Object cf = pmf.getConnectionFactory();
        Assert.assertTrue("The connection factory is not a DataSource",
            cf instanceof DataSource);
        try {
            ((DataSource) cf).getConnection().close();
        } catch (SQLException e) {
            fail("Impossible to get a connection on the factory " + cf);
        }
    }
}