/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.basic;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.mapper.lib.Object2StringSerializer;
import org.objectweb.util.monolog.api.BasicLevel;

import java.io.File;
import java.io.IOException;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestObject2StringSerializer extends SpeedoTestHelper {

	String output;

	public TestObject2StringSerializer(String s) {
		super(s);
		output = System.getProperty("out.test", File.separatorChar + "tmp");
		logger.log(BasicLevel.DEBUG, "output=" + output);
	}

	protected String getLoggerName() {
		return "org.objectweb.speedo.runtime.basic.TestObject2StringSerializer";
	}

	public void testSerialization() {
		testSerialization("fdfdmfd,ng,nbdlfdfgkf,n");
	}
	public void testSerializationBigObject() {
		StringBuffer sb = new StringBuffer(239000);
		for(int i=0; i<50000; i++) {
			sb.append(i);
		}
		testSerialization(sb.toString());
	}

	public void testSerialization(Object o) {
		String value = null;
		try {
			value = Object2StringSerializer.serialize(o);
			assertNotNull("Null string", value);
		} catch (IOException e) {
			logger.log(BasicLevel.ERROR, "The serialization fails", e);
			fail("The serialization fails");
		}
		try {
			Object o2 = Object2StringSerializer.deserialize(value);
			assertEquals("Objects not equal", o, o2);
		} catch (IOException e) {
			logger.log(BasicLevel.ERROR, e.getMessage(), e);
			fail("The deserialization fails");
		} catch (ClassNotFoundException e) {
			logger.log(BasicLevel.ERROR, e.getMessage(), e);
			fail("The deserialization fails");
		}
	}
	private void testSerializationIntoFile(Object o, String fileName) {
		String jdoFile = fileName + ".jdo";
		String className = Object2StringSerializer.descFileName2ClassName(jdoFile);
		try {
			Object2StringSerializer.serialize(
					output,
					jdoFile,
					o,
					logger);
		} catch (Exception e) {
			logger.log(BasicLevel.ERROR, e.getMessage(), e);
			fail("The class creation fail");
		}
		int i = 0;
		try {
			Class c = Class.forName(className);
			i = 1;
			Object2StringSerializer o2ss = (Object2StringSerializer) c.newInstance();
			i = 2;
			Object2StringSerializer.deserialize(o2ss.getSerializedObject());
		} catch (Exception e) {
			e.printStackTrace();
			logger.log(BasicLevel.ERROR, e.getMessage(), e);
			switch(i) {
			case 0:
				fail("The class loading fail");
				break;
			case 1:
				fail("The class instanciation fail");
				break;
			default:
				fail("The deserialization fail");
				break;
			}
		} finally {
			String fn = Object2StringSerializer.descFileName2ClassName(fileName + ".jdo");
			className = className.replace('.', '/');
			File f = new File(output, fn + ".class");
			if (f.exists()) {
				logger.log(BasicLevel.DEBUG,  "Remove the file " + f.getAbsolutePath());
				f.delete();
			}
		}
	}

	public void testSerializationIntoFile() {
		testSerializationIntoFile("fdfdmfd,ng,nbdlfdfgkf,n", "testSerializationIntoFile");
	}
	public void testSerializationIntoFileBigObject() {
		StringBuffer sb = new StringBuffer(239000);
		for(int i=0; i<50000; i++) {
			sb.append(i);
		}
		testSerializationIntoFile(sb.toString(), "testSerializationIntoFileBigObject");
	}
}
