/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.runtime.detach;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.jdo.FetchPlan;
import javax.jdo.JDOException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import junit.framework.Assert;

import org.objectweb.speedo.SpeedoTestHelper;
import org.objectweb.speedo.api.ExceptionHelper;
import org.objectweb.speedo.pobjects.detach.Coach;
import org.objectweb.speedo.pobjects.detach.Player;
import org.objectweb.speedo.pobjects.detach.Team;
import org.objectweb.speedo.pobjects.detach.groupama.A;
import org.objectweb.speedo.pobjects.detach.groupama.B;
import org.objectweb.speedo.pobjects.detach.groupama.C;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Try to reproduce the bug of Unilog Groupama:
 * when performing two detachAll on a query result, problem with the cache: 
 * PName already bound to another object 
 * @author Y.Bersihand
 */
public class TestDetachAll extends SpeedoTestHelper {

	public TestDetachAll(String s) {
		super(s);
	}

	protected String getLoggerName() {
		return LOG_NAME + ".rt.detach.TestDetachAll";
	}
	
	/**
	 * Test the detachall method
	 */
	public void testDetachAllOnce() {
		logger.log(BasicLevel.DEBUG, "***************testDetachAllOnce*****************");
		createABC();
		//detach all once
		try{
			Collection as = detachAs();
			Iterator it1 = as.iterator();
			logger.log(BasicLevel.DEBUG, "1st detached:");
			while(it1.hasNext()){
				A a = (A) it1.next();
				logger.log(BasicLevel.DEBUG, a.toString());
			}
		}
		catch (Exception e) {
			logger.log(BasicLevel.DEBUG, e.getMessage());
		}
	}
	
	public void testDetachAllTwice() {
		logger.log(BasicLevel.DEBUG, "***************testDetachAllTwice*****************");
		//detach all twice
		try{
			Collection as = detachAs();
			Iterator it1 = as.iterator();
			logger.log(BasicLevel.DEBUG, "2nd detached:");
			while(it1.hasNext()){
				A a = (A) it1.next();
				logger.log(BasicLevel.DEBUG, a.toString());
			}
		}
		catch (Exception e) {
			logger.log(BasicLevel.DEBUG, e.getMessage());
		}
	}
	
	public void testRemovingOfPersistentObject() {
        PersistenceManager pm = pmf.getPersistenceManager();
        try {
            Class[] cs = new Class[]{A.class, B.class, C.class, 
            		Coach.class, Player.class, Team.class};
	        pm.currentTransaction().begin();
            for(int i=0; i<cs.length; i++) {
                Query query = pm.newQuery(cs[i]);
                Collection col = (Collection) query.execute();
                Iterator it = col.iterator();
                while(it.hasNext()) {
                    Object o = it.next();
                    Assert.assertNotNull("null object in the query result"
                        + cs[i].getName(), o);
                    pm.deletePersistent(o);

                }
                query.close(col);
            }
	        pm.currentTransaction().commit();
        } catch (JDOException e) {
            Exception ie = ExceptionHelper.getNested(e);
            logger.log(BasicLevel.ERROR, "", ie);
            fail(ie.getMessage());
        } finally {
            pm.close();
        }
    }
	private void createPlayers(){
		Team t = new Team("Bordeaux",null,null);
		Collection players = new ArrayList();
		Player p1 = new Player("p1", t, 25);
		players.add(p1);
		Player p2 = new Player("p2", t, 32);
		players.add(p2);
		t.setPlayers(players);
		Coach c = new Coach("c1", 5, t);
		t.setCoach(c);
		
		Team t2 = new Team("Nantes", null, null);
		Collection players2 = new ArrayList();
		Player p3 = new Player("p3", t2, 25);
		players2.add(p3);
		Player p4 = new Player("p4", t2, 32);
		players2.add(p4);
		t2.setPlayers(players2);
		Coach c2 = new Coach("c2", 5, t2);
		t2.setCoach(c2);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		logger.log(BasicLevel.DEBUG, "make persistent the team " + t.toString());
		pm.makePersistent(t);
		pm.makePersistent(t2);
		pm.currentTransaction().commit();
	}
	
	private Collection detachTeams() throws Exception{
		PersistenceManager pm = pmf.getPersistenceManager();
		try{
			Query query = pm.newQuery(Team.class);
			Collection results = (Collection)query.execute();
			logger.log(BasicLevel.DEBUG, "Teams:");
			Iterator it = results.iterator();
			while(it.hasNext()){
				Team t = (Team) it.next();
				logger.log(BasicLevel.DEBUG, t.toString());
			}
			Collection detachedTeams = (Collection) pm.detachCopyAll(results);
			query.closeAll();
			return detachedTeams;
		}
		catch(Exception e){
			logger.log(BasicLevel.DEBUG, e.toString());
			return null;
		}
		finally{
			try {
				logger.log(BasicLevel.DEBUG, "pm closed");
			 	pm.close();
			}
			catch (Exception e) {
				logger.log(BasicLevel.DEBUG, e.getMessage());
			} 
		}
	}
	
	private void createABC(){
		A a1 = new A(1);
		A a2 = new A(2);
		A a3 = new A(3);
		
		C c1 = new C(1,1,1);
		C c2 = new C(2,2,2);
		C c3 = new C(3,3,3);
		C c4 = new C(4,4,4);
		C c5 = new C(5,5,5);
		C c6 = new C(6,6,6);
		
		B b1 = new B(1, c1);
		B b2 = new B(2, c2);
		B b3 = new B(3, c3);
		B b4 = new B(4, c4);
		B b5 = new B(5, c5);
		B b6 = new B(6, c6);
		
		a1.addB(b1);
		a1.addB(b2);
		a1.addB(b3);
		
		a2.addB(b4);
		a2.addB(b5);
		
		a3.addB(b6);
		
		PersistenceManager pm = pmf.getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(a1);
		pm.makePersistent(a2);
		pm.makePersistent(a3);
		pm.currentTransaction().commit();
	}
	
	private Collection detachAs(){
		PersistenceManager pm = pmf.getPersistenceManager();
		FetchPlan fetchPlan = pm.getFetchPlan(); 
	    fetchPlan.addGroup("AList").removeGroup("default");
		try{
			Query query = pm.newQuery(A.class);
			Collection results = (Collection)query.execute();
			logger.log(BasicLevel.DEBUG, "A:");
			Iterator it = results.iterator();
			while(it.hasNext()){
				A a = (A) it.next();
				logger.log(BasicLevel.DEBUG, "" + a.toString());
			}
			Collection detachedAs = (Collection) pm.detachCopyAll(results);
			query.closeAll();
			return detachedAs;
		}
		catch(Exception e){
			logger.log(BasicLevel.DEBUG, e.toString());
			return null;
		}
		finally{
			try {
				logger.log(BasicLevel.DEBUG, "pm closed");
			 	pm.close();
			}
			catch (Exception e) {
				logger.log(BasicLevel.DEBUG, e.getMessage());
			} 
		}	
	}
	
}
