/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.runtime.jca;

import javax.naming.spi.InitialContextFactory;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.Name;
import javax.naming.NamingEnumeration;
import javax.naming.NameParser;
import java.util.Hashtable;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class SimpleNamingManager implements InitialContextFactory {

    private static SimpleContext ictx;

    public SimpleNamingManager() {
    }

    public Context getInitialContext(Hashtable environment)
            throws NamingException {
        if (ictx == null) {
            ictx = new SimpleContext(environment);
        } else {
            ictx.context.putAll(environment);
        }
        return ictx;
    }

    class SimpleContext implements Context {

        public Hashtable context;

        public SimpleContext(Hashtable context) {
            this.context = context;
        }

        private String name2String(Name n) {
            StringBuffer sb = new StringBuffer("");
            String sep = "";
            for(int i=0; i<n.size(); i++) {
                sb.append(sep);
                sep = ".";
                sb.append(n.get(i));
            }
            return sb.toString();
        }

        // IMPLEMENTATION OF THE Context INTERFACE //
        //-----------------------------------------//

        public Object lookup(Name name) throws NamingException {
            return context.get(name2String(name));
        }

        public Object lookup(String name) throws NamingException {
            return context.get(name);
        }

        public void bind(Name name, Object obj) throws NamingException {
            String n = name2String(name);
            if (context.containsKey(n)) {
                throw new NamingException(n + " already bound: " + context.get(n));
            } else {
                context.put(n, obj);
            }
        }

        public void bind(String n, Object obj) throws NamingException {
            if (context.containsKey(n)) {
                throw new NamingException(n + " already bound: " + context.get(n));
            } else {
                context.put(n, obj);
            }
        }

        public void rebind(Name name, Object obj) throws NamingException {
            context.put(name2String(name), obj);
        }

        public void rebind(String n, Object obj) throws NamingException {
            context.put(n, obj);
        }

        public void unbind(Name name) throws NamingException {
            context.remove(name2String(name));
        }

        public void unbind(String name) throws NamingException {
            context.remove(name);
        }

        public void rename(Name oldName, Name newName) throws NamingException {
            bind(newName, lookup(oldName));
            unbind(oldName);
        }

        public void rename(String oldName, String newName) throws NamingException {
            bind(newName, lookup(oldName));
            unbind(oldName);
        }

        public NamingEnumeration list(Name name) throws NamingException {
            return null;
        }

        public NamingEnumeration list(String name) throws NamingException {
            return null;
        }

        public NamingEnumeration listBindings(Name name) throws NamingException {
            return null;
        }

        public NamingEnumeration listBindings(String name) throws NamingException {
            return null;
        }

        public void destroySubcontext(Name name) throws NamingException {
        }

        public void destroySubcontext(String name) throws NamingException {
        }

        public Context createSubcontext(Name name) throws NamingException {
            return null;
        }

        public Context createSubcontext(String name) throws NamingException {
            return null;
        }

        public Object lookupLink(Name name) throws NamingException {
            return null;
        }

        public Object lookupLink(String name) throws NamingException {
            return null;
        }

        public NameParser getNameParser(Name name) throws NamingException {
            return null;
        }

        public NameParser getNameParser(String name) throws NamingException {
            return null;
        }

        public Name composeName(Name name, Name prefix) throws NamingException {
            return null;
        }

        public String composeName(String name, String prefix)
                throws NamingException {
            return null;
        }

        public Object addToEnvironment(String propName, Object propVal)
                throws NamingException {
            return null;
        }

        public Object removeFromEnvironment(String propName)
                throws NamingException {
            return null;
        }

        public Hashtable getEnvironment() throws NamingException {
            return context;
        }

        public void close() throws NamingException {
        }

        public String getNameInNamespace() throws NamingException {
            return null;
        }
    }
}
