/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Authors: S.Chassande-Barrioz.
 * Created on 16 fevr. 2005
 *
 */
package org.objectweb.speedo.tools;

import java.util.regex.Pattern;

import junit.framework.TestCase;

/**
 * 
 *
 * @author S.Chassande-Barrioz
 */
public class TestStringReplace extends TestCase {

    public TestStringReplace() {
        this("TestStringReplace");
    }
    public TestStringReplace(String name) {
        super(name);
    }
    
    public static void main(String[] args) {
        junit.textui.TestRunner.run(TestStringReplace.class);
    }

    public void testReplaceChar() {
        assertEquals("org/objectweb/speedo/Foo",
                StringReplace.replaceChar('.', '/', 
                        "org.objectweb.speedo.Foo"));
        assertEquals("org\\objectweb\\speedo\\Foo",
                StringReplace.replaceChar('.', '\\', 
                        "org.objectweb.speedo.Foo"));
    }

    public void testToJavaPattern() {
        String s = "org.objectweb.s?peedo.*.*Foo*";
        assertEquals("org\\.objectweb\\.s.?peedo\\..*\\..*Foo.*", 
                StringReplace.toJavaPattern(s));
    }
    public void testPatterns() {
        assertPattern("org", "org", true);
        assertPattern("org.objectweb.*", "org.objectweb.toto", true);
        assertPattern("org.objectweb.*", "org.objectwe.toto", false);
        assertPattern("org.objectweb.*#*", "org.objectweb.Toto#", true);
        assertPattern("org.objectweb.*#*", "org.objectweb.Toto#f2", true);
        assertPattern("org.objectweb.*#*", "org.objectwe.Toto#", false);
        assertPattern("org.objectweb.*#f1", "org.objectweb.Toto#f1", true);
        assertPattern("org.objectweb.*#f1", "org.objectweb.dgf.sdfsgdf.Toto#f1", true);
        assertPattern("org.objectweb.*#f1", "org.objectweb.#f1", true);
        assertPattern("org.objectweb.*#f1", "org.objectweb.Toto#f1", true);
        assertPattern("org.objectweb.*#f1", "org.objectweb.Toto#f2", false);
        assertPattern("org.objectweb.*#f?", "org.objectweb.Toto#f2", true);
        assertPattern("org.objectweb.*#f?", "org.objectweb.Toto#f3", true);
        
    }
    
    private void assertPattern(String pattern, String exp, boolean match) {
        assertEquals(match, 
                Pattern.compile(StringReplace.toJavaPattern(pattern))
                	.matcher(exp).matches());
    }
}
