/*
 * Speedo: an implementation of JDO compliant personality on top of JORM
 * generic I/O sub-system. Copyright (C) 2001-2004 France Telecom R&D
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Release: 1.0
 * 
 * Created on Apr 9, 2004 @author fmillevi@yahoo.com
 *  
 */
package org.objectweb.speedo.j2eedo;

import javax.jdo.JDOException;
import javax.jdo.JDOFatalException;

import org.objectweb.speedo.j2eedo.common.HTTPTools;
import org.objectweb.speedo.j2eedo.web.ApplicationServlet;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class defines junit application tests. The different actions are
 * performed throulaunch an HTTP request on a J2EE application server.
 * <p>
 * <b>It's assume that the j2eedo application have been deployed on it.</b>
 * 
 * @author fmillevi@yahoo.com
 * @see TestJ2eedoApplication
 */
public class TestJ2eedoJ2ee extends TestJ2eedoApplication {
	protected final String TESTJ2EE = getLoggerName() + ".j2ee";
	protected final String URL = getLoggerName() + ".url";
	protected final String HTTPPARAMS = getLoggerName() + ".httpParams";

	protected String url = null;
	protected String httpParams = null;

	/**
	 * Empty constructor, call the TestJ2eedoApplication constructor and gets
	 * the specific parameters for J2EE application test
	 * 
	 * @param s
	 *            name of class
	 */
	public TestJ2eedoJ2ee(String s) {
		super(s);
		this.url =
			System.getProperty(this.URL, "http://localhost:9000/j2eedo/");
		this.httpParams =
			System.getProperty(
				this.HTTPPARAMS,
				ApplicationServlet.WITH_TRANSACTION_PARAMETER + "=true");
	}

	protected void setUp() {
	    super.setUp();
		if (null == this.url)
			return;
	    try {
            logger.log(BasicLevel.INFO, HTTPTools.getURL(
            		this.url + "evictall.do",
            		this.httpParams
            			+ "&"
            			+ ApplicationServlet.PARAMETER_ACTION
            			+ "=evictall",
            		"GET",
            		logger));
        } catch (Exception e) {
            logger.log(BasicLevel.ERROR, "");
        }
	    
	}
	protected void displayTestParameters() {
		if (!logger.isLoggable(BasicLevel.INFO))
			return;
		super.displayTestParameters();

		logger.log(
			BasicLevel.INFO,
			"\n\tJ2EE application URL: "
				+ this.url
				+ "\n\tHTTP default parameters : "
				+ this.httpParams);
	}

	/**
	 * This method executes the action pass as parameters throught an HTTP get
	 * URL call.
	 * 
	 * @param action
	 *            defines the type of action to be performed.
	 * @exception Exception
	 *                in case of error detected during HTTP call
	 * @exception
	 * @see org.objectweb.speedo.j2eedo.bo.DatabaseImpl
	 * @see org.objectweb.speedo.j2eedo.TestJ2eedoApplication#performMethod(java.lang.String)
	 */
	protected String performMethod(String action)
		throws Exception, JDOException, JDOFatalException {
		String str = "";
		try {
			return HTTPTools.getURL(
				this.url + action + ".do",
				this.httpParams
					+ "&"
					+ ApplicationServlet.PARAMETER_ACTION
					+ "="
					+ action,
				"GET",
				logger);
		} catch (Exception e) {
			logger.log(
				BasicLevel.ERROR,
				"HTTP:The call \""
					+ this.url
					+ action
					+ ".do?"
					+ this.httpParams
					+ "&"
					+ ApplicationServlet.PARAMETER_ACTION
					+ "="
					+ action
					+ "\" throws Exception ",
				e);
			assertTrue(
				"HTTP:The call \""
					+ this.url
					+ action
					+ ".do?"
					+ this.httpParams
					+ "&"
					+ ApplicationServlet.PARAMETER_ACTION
					+ "="
					+ action
					+ "\" throws Exception :\n"
					+ e.toString(),
				false);
			throw e;
		}
	}

	/**
	 * This method calls action of the DatabaseImpl class throught an HTTP call
	 * using an <b>EJB session container</b>
	 * 
	 * @see org.objectweb.speedo.j2eedo.bo.DatabaseImpl
	 */
	public void testHTTPCallSessionContainer() {
		if (this.simpleTestOnly)
			return;
		if (null == this.url)
			return;
		this.httpParams =
			ApplicationServlet.WITH_TRANSACTION_PARAMETER
				+ "=false&"
				+ ApplicationServlet.TYPE_CONTAINER_PARAMETER
				+ "=true&"
				+ ApplicationServlet.JDOTX
				+ "=false";
		this.perform();
	}

	/**
	 * This method calls action of the DatabaseImpl class throught an HTTP call
	 * using an <b>EJB session container</b>. , the servlet <b>gets the
	 * persistence manager</b>
	 * 
	 * @see org.objectweb.speedo.j2eedo.bo.DatabaseImpl
	 */
	public void testHTTPCallSessionContainerWithJdoTx() {
		if (this.simpleTestOnly)
			return;
		if (null == this.url)
			return;
		this.httpParams =
			ApplicationServlet.WITH_TRANSACTION_PARAMETER
				+ "=false&"
				+ ApplicationServlet.TYPE_CONTAINER_PARAMETER
				+ "=true&"
				+ ApplicationServlet.JDOTX
				+ "=true";
		this.perform();
	}

	/**
	 * This method calls action of the DatabaseImpl class throught an HTTP call
	 * <b>without any transaction</b> is started, the servlet <b>gets the
	 * persistence manager</b>
	 * 
	 * @see org.objectweb.speedo.j2eedo.bo.DatabaseImpl
	 */
	public void testHTTPWithTransWithJdoTx() {
		if (this.simpleTestOnly)
			return;
		if (null == this.url)
			return;
		this.httpParams =
			ApplicationServlet.WITH_TRANSACTION_PARAMETER
				+ "=true&"
				+ ApplicationServlet.JDOTX
				+ "=true";
		this.perform();
	}
}