/*
 * Speedo: an implementation of JDO compliant personality on top of JORM
 * generic I/O sub-system. Copyright (C) 2001-2004 France Telecom R&D
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Release: 1.0
 * 
 * Created on 27 feb. 2004 @author franck.milleville@cgey.com
 *  
 */
package org.objectweb.speedo.j2eedo.database;

import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

/**
 * This class define the Java class to access to the employee's data
 * @author franck.milleville @cgey.com
 *  
 */
public class Employee implements DatabaseObjectInterface {
	private long empid;
	private String firstname;
	private String lastname;
	private Address address;
	private Date birthdate;
	private Date hiredate;
	private float salary;
	private Department department;
	private Employee manager;
	private Collection slaves;
	private Collection projects; //element:Project


	/**
	 * @param firstname
	 * @param lastname
	 * @param birthdate
	 * @param department
	 */
	public Employee(
		String firstname,
		String lastname,
		Date birthdate,
		Department department) {
		super();
		this.firstname = firstname;
		this.lastname = lastname;
		this.birthdate = birthdate;
		this.department = department;
		this.hiredate = new Date();
	}
	/**
	 * @param firstname
	 * @param lastname
	 * @param address
	 * @param birthdate
	 * @param hiredate
	 * @param salary
	 * @param department
	 * @param manager
	 * @param projects
	 */
	public Employee(
		String firstname,
		String lastname,
		Address address,
		Date birthdate,
		Date hiredate,
		float salary,
		Department department,
		Employee manager,
		Collection projects) {
		super();
		this.firstname = firstname;
		this.lastname = lastname;
		this.address = address;
		this.birthdate = birthdate;
		this.hiredate = hiredate;
		this.salary = salary;
		this.department = department;
		this.manager = manager;
		this.projects = projects;
	}
	/**
	 * Empty constructor
	 */
	public Employee() {
		super();
	}	
	public String getAsString() {
		StringBuffer sb = new StringBuffer();
		sb.append("\nEmployee\t(empid:").append(this.empid);
		sb.append(" department:").append(this.department.getName());
		sb.append(")\n\tFirst name:").append(this.firstname);
		sb.append("\tlast name:").append(this.lastname);
		sb.append("\tbirthdate:").append(this.birthdate);
		sb.append("\tmanager:");
		if (null != this.manager)
			sb.append(this.manager.getLastname());
		else
			sb.append("No manager");
		sb.append("\tsalary:").append(this.salary);
		sb.append("\thire date:").append(this.hiredate);
		sb.append("\n\tadsress:").append(this.address.getAsString());
		if (this.projects != null) {
			Iterator i = projects.iterator();
			if (i.hasNext()) {
				sb.append("\tMember of projects:");
			}
			while (i.hasNext()) {
				sb.append(((Project) i.next()).getName()).append(" ");
			}
		}
		return sb.toString();
	}
	/**
	 * @see org.objectweb.speedo.j2eedo.database.DatabaseObjectInterface#getId()
	 */
	public long getId() {
		return this.empid;
	}

	/**
	 * @return Returns the empid.
	 */
	public long getEmpid() {
		return empid;
	}

	/**
	 * @return Returns the hiredate.
	 */
	public Date getHiredate() {
		return hiredate;
	}

	/**
	 * @param hiredate
	 *            The hiredate to set.
	 */
	public void setHiredate(Date hiredate) {
		this.hiredate = hiredate;
	}

	/**
	 * @return Returns the lastname.
	 */
	public String getLastname() {
		return lastname;
	}

	/**
	 * @param lastname
	 *            The lastname to set.
	 */
	public void setLastname(String lastname) {
		this.lastname = lastname;
	}

	/**
	 * @return Returns the manager.
	 */
	public Employee getManager() {
		return manager;
	}

	/**
	 * @param manager
	 *            The manager to set.
	 */
	public void setManager(Employee manager) {
		this.manager = manager;
	}

	/**
	 * @return Returns the projects.
	 */
	public Collection getProjects() {
		return projects;
	}

	/**
	 * @param projects
	 *            The projects to set.
	 */
	public void setProjects(Collection projects) {
		this.projects.addAll(projects);
	}

	/**
	 * @return Returns the salary.
	 */
	public float getSalary() {
		return salary;
	}

	/**
	 * @param salary
	 *            The salary to set.
	 */
	public void setSalary(float salary) {
		this.salary = salary;
	}
	/**
	 * @return Returns the address.
	 */
	public Address getAddress() {
		return address;
	}

	/**
	 * @param address
	 *            The address to set.
	 */
	public void setAddress(Address address) {
		this.address = address;
	}

	/**
	 * @return Returns the birthdate.
	 */
	public Date getBirthdate() {
		return birthdate;
	}

	/**
	 * @param birthdate
	 *            The birthdate to set.
	 */
	public void setBirthdate(Date birthdate) {
		this.birthdate = birthdate;
	}

	/**
	 * @return Returns the department.
	 */
	public Department getDepartment() {
		return department;
	}

	/**
	 * @param department
	 *            The department to set.
	 */
	public void setDepartment(Department department) {
		this.department = department;
	}

	/**
	 * @return Returns the firstname.
	 */
	public String getFirstname() {
		return firstname;
	}

	/**
	 * @param firstname
	 *            The firstname to set.
	 */
	public void setFirstname(String firstname) {
		this.firstname = firstname;
	}
}